######################################
#                                    #
#   Survival analysis workshop       #
#   January 2012                     #
#   Example code                     #
#                                    #
######################################


##
 # First, let us consider a fairly trivial example with no complications. 
 # (Assumes that the data files are in your current working directory)
 ##


full <- read.csv("RollDice_complete.csv")
### In this example, I've taken 100 dice and rolled each one until it turned up a 6, at which point it was dead. 
#   Each die was observed until it died (i.e., it the process was fully observed, hence the name "full")
#   I'm going to call each die an animal so I can avoid talking about dice dying... 
#   I just recorded how many rolls until each animal died

### We can get a look at the distribution of rolls until death using the table() command

table(full)
#     Death
# Roll  1
#   1  20
#   2  11
#   3  10
#   4  10
#   5   9
#   6   7
#   7   5
#   8  10
#   9   4
#   10  3
#   11  3
#   12  1
#   13  1
#   16  1
#   17  1
#   19  3
#   23  1

mean(full$Roll)
# [1] 5.51

### The mean number of rolls until death is not far off from the 1/(1/6) we would expect 
#   (Since each roll each animal has a 1 in 6 chance of death, it should take on average 6 rolls for an animal to die)
#   Because there may be a few individuals that take a really long time to go, the median, which is less affected by
#   outliers, is often a better description of the average

median(full$Roll)
# [1] 4

### Why isn't the median also close to 1/(1/6)? 
#   With a constant rate of death, the distribution of times to death follows an exponential distribution, 
#   and while the mean of the exponential is 1/rate, the median is 1/rate*ln(2), which in our case is 4.159. 
#   We're pretty darn close to perfect!



###
  # Now, let us consider a slightly more realistic and complex example of how we might end up with survival data.
  # The underlying process is the same (I rolled dice), but there are two changes:
  #  1) I only rolled the dice 10 times (i.e., my experiment was only 10 rolls). 
  #     Animals alive after the tenth roll were given values of Roll = 10 and Death = 0 (i.e., they did not die).
  #     These animals were "censored". 
  #  2) On the second and seventh rolls (days), I removed some dice/animals from the population.  
  #     These animals were censored, too. 
  #     This might have been to destructively sample a few individuals here there, or it could have happened accidentally.
  ###

cens <- read.csv("RollDice_censored.csv")
table(cens)
#     Death
# Roll  0  1
#   1   0 16
#   2   3 16
#   3   0 10
#   4   0 10
#   5   0  5
#   6   0  8
#   7   3  7
#   8   0  4
#   9   0  4
#   10 13  1


### So here is where things get tricky. We want to know the average time to death, 
#   but should we include those that were censored (either in the middle or at the end of the experiment) in our average? #   If so, we are pretending that they died on the tenth roll, but they didn't. 

mean(cens$Roll)
# [1] 4.75
median(cens$Roll)
# [1] 4

### You may have noticed that this median is exactly what we had in the previous example. 
#   So maybe censoring is not such a big deal after all. 
#   Actually, it will depend a great deal on how many animals are censored and when, 
#   as well as the distribution of times to death. 
#   I wouldn't trust this this happy accident will repeat itself in your data!


### If we exclude those censored animals we ignore the fact that there were there and did not die in the 10 day interval.
#   We are essentially throwing away data!

mean(subset(cens, Death == 1)$Roll)
# [1] 3.925926
median(subset(cens, Death == 1)$Roll)
# [1] 3

### Either way, we bias our estimates, sometimes by quite a lot. 



# So let's dig in, then.
# First, load the survival package
library(survival)

### Let's calculate the Kaplan-Meier median time to death and confidence interval
#   First, we need to convert the data to a "Surv" object, which includes the censoring information.
#   We use the function, Surv(). It takes a time-to-event and an indicator of whether the event occured.
#   In [R] the event (e.g., death) is coded as 1, a censored individual is 0.
#   Note: the coding may differ between stats packages.
#   Note: Sometimes we can only account for the fates of individuals over intervals, 
#         e.g., from one month to the next, and do not know exactly when the event occured. 
#         This package can accomodate this and much more. See the help files and help online for more. )

s1 <- Surv(cens$Roll, cens$Death) # note the capital "S"
s1
#   [1]  1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   2   2   2   2   2   2   2   2   2 
#  [26]  2   2   2   2   2   2   2   2+  2+  2+  3   3   3   3   3   3   3   3   3   3   4   4   4   4   4 
#  [51]  4   4   4   4   4   5   5   5   5   5   6   6   6   6   6   6   6   6   7   7   7   7   7   7   7 
#  [76]  7+  7+  7+  8   8   8   8   9   9   9   9  10  10+ 10+ 10+ 10+ 10+ 10+ 10+ 10+ 10+ 10+ 10+ 10+ 10+

### This looks just like the list of times to death, only now the censored individuals have a plus after the
#   time when they were censored to indicate that they survived for at least x rolls.


### Then we use the survfit() function to calculate the K-M survival estimate for our data.
#   This function takes a Surv object on the left of the tilde and the grouping variable on the right 
#   This sort of construction is the formula interface. 
#   It is used here because this function was set up to compare survival among groups. 
#   We'll get there, but for now, we're saying that there are no groups, just an intercept (x ~ 1)

survfit(s1~1) 
# Call: survfit(formula = s2 ~ 1)
# 
# records   n.max n.start  events  median 0.95LCL 0.95UCL 
#     100     100     100      81       4       3       6 

### Notice that we recovered the "correct" median of ~4 rolls
    
### We can get more information using the summary() function
#   Specifically, it returns the survival probability, S(t), for each time
#   as well as the standard error of S(t) and the 95% CI

summary(survfit(s1~1))    
# Call: survfit(formula = s2 ~ 1)
# 
#  time n.risk n.event survival std.err lower 95% CI upper 95% CI
#     1    100      16    0.840  0.0367       0.7711        0.915
#     2     84      16    0.680  0.0466       0.5945        0.778
#     3     65      10    0.575  0.0498       0.4855        0.682
#     4     55      10    0.471  0.0506       0.3814        0.581
#     5     45       5    0.418  0.0501       0.3310        0.529
#     6     40       8    0.335  0.0480       0.2527        0.443
#     7     32       7    0.262  0.0448       0.1870        0.366
#     8     22       4    0.214  0.0425       0.1450        0.316
#     9     18       4    0.166  0.0391       0.1050        0.264
#    10     14       1    0.155  0.0381       0.0953        0.251


### Plotting K-M survival curves is easy. 
#   By default, with just one group, the function also plots the 95% CI
plot(survfit(s1~1))
plot(survfit(s1~1), conf.int = F)
plot(survfit(s1~1), conf.int = F, xlab = "Roll to death", ylab = "Proportion surviving")


### As an aside, we can combine the two datasets, analyze them together, and make a plot of showing both curves
cens$observations <- "Censored"
full$observations <- "Full"
dice <- rbind(full, cens)

b <- survfit(Surv(Roll, Death) ~ observations, data = dice)

plot(b, lty = 1:2, col = c("blue", "red")) 
legend(10, .9, c("Censored", "Fully observed"), lty = 1:2, col = c("blue", "red"), bty = "n")


###
  # Usually we want to compare groups (or, later, individuals with different covariates)
  # so let's include another group with a different survival curve
  # This new dataset includes both the dice, from before, and coins that were flipped until a head came up
  # Heads equals dead was the rule
  ###

dicecoin <- read.csv("DiceCoin.csv")

### The K-M estimates are produced the same way as before, but I am changing two things:
#    1) I've included the call to create the Surv object inside of the other function
#       You assign the Surv object to a variable as above (e.g., s2), but I think this is more transparent
#    2) Now we have a grouping variable on the right-hand side, "Type"

dc <- survfit(Surv(Time, Death) ~ Type, data = dicecoin)
dc
# Call: survfit(formula = Surv(Time, Death) ~ Type, data = dicecoin)
# 
#           records n.max n.start events median 0.95LCL 0.95UCL
# Type=Coin     100   100     100     97      2       1       2
# Type=Die      100   100     100     81      4       3       6

### Note the median times differ and the 95%CIs do not overlap. This is one way of waying they are different.

### Plot the data in a simple way
plot(dc)

### Or slightly fancier with different line types (lty) and colors (col)
plot(dc, lty = 1:2, col = c("blue", "red"), xlab = "Time in flips or rolls", ylab = "Proportion alive")

### And then add a legend
legend(4, .9, c("Coins", "Dice"), lty = 1:2, col = c("blue", "red"), bty = "n")



### Here is the code for the Log-rank test (aka Mantel or Mantel-Haenszel test)
#   It uses the same formulat interface as above.

survdiff(Surv(Time, Death) ~ Type, data = dicecoin)
# Call:
# survdiff(formula = Surv(Time, Death) ~ Type, data = dicecoin)
# 
#             N Observed Expected (O-E)^2/E (O-E)^2/V
# Type=Coin 100       97     57.7      26.7      65.3
# Type=Die  100       81    120.3      12.8      65.3
# 
#  Chisq= 65.3  on 1 degrees of freedom, p= 6.66e-16 

### The output include ths sample sizes, the number of events that were Observed, the 
#   Expected number of events, assuming there is no different between groups, and the 
#   Chi-square-like (observed-expected)^2/expected, as well as the version standardized
#   by the variance, V, which the test is built on. This is compared to the Chi-square distribution
#   It is pretty clear that these groups differ in number suriving at the various times!
 


### This code performs the Cox proportional hazard test
#   Again, this assumes an underlying baseline hazard and then determines
#   whether different groups or individuals with different covariates have
#   a proportionately higher or lower hazard
#   It also uses the formula interface, as before

dc.cox <- coxph(Surv(Time, Death) ~ Type, data = dicecoin) 
summary(dc.cox)
# Call:
# coxph(formula = Surv(Time, Death) ~ Type, data = dicecoin)
# 
#   n= 200, number of events= 178 
# 
#            coef exp(coef) se(coef)      z Pr(>|z|)    
# TypeDie -1.3479    0.2598   0.1725 -7.814 5.55e-15 ***
# ---
# Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1 
# 
#         exp(coef) exp(-coef) lower .95 upper .95
# TypeDie    0.2598      3.849    0.1853    0.3643
# 
# Concordance= 0.677  (se = 0.03 )
# Rsquare= 0.27   (max possible= 1 )
# Likelihood ratio test= 63.07  on 1 df,   p=1.998e-15
# Wald test            = 61.06  on 1 df,   p=5.551e-15
# Score (logrank) test = 67.91  on 1 df,   p=2.22e-16

### Like a regression, it returns the coefficient(s) and test statistics for them.
#   (There is no "intercept", though, because that is wrapped up in the baseline hazard)
#   Unlike most regressions, the coefficients need to be exponentiated to tell you about
#   increases or decreases in hazard from the baseline. 
#   This is called the "hazard ratio"
#   Here the hazard ratio is 0.2598, which means that the hazard for a Die is ~1/4 that of the Coins
#   Note: most people report the P-value associated with the Likelihood ratio test. 

### It is possible to extract the estimated baseline hazard, H(t), although it is not always very informative.
#   Actually, what basehaz is the cummulative baseline hazard (it always increases)

basehaz( dc.cox )
#       hazard time
# 1  0.3401289    1
# 2  0.7302868    2
# 3  1.0999080    3
# 4  1.4558872    4
# 5  1.6903325    5
# 6  2.1360030    6
# 7  2.6457717    7
# 8  3.0297384    8
# 9  3.5075668    9
# 10 3.6477072   10

### It is possible to calculate the differences in the cummulative hazard to provide an estimate of h(t)
bh <- basehaz( dc.cox)
diff(bh$hazard)
### OR
diff( basehaz( dc.cox )$hazard )
# [1] 0.3901579 0.3696212 0.3559792 0.2344454 0.4456705 0.5097687 0.3839668 0.4778284 0.1401404

### The mean of these hazards is not that different from the mean of 1/2 and 1/6, which are the True
#   hazards of the Coins and Dice, respectively
mean(  diff( basehaz( dc.cox )$hazard )  )
# [1] 0.3675087


### Lastly, let use parametric survival analyses to analyze these data
#   We have a pretty good idea that the underlying hazard is constant, around 1/2 for coins and 1/6 for dice
#   This interface is like regression, as before, but now we need to specify the distribution

dc.aft <- survreg(Surv(Time, Death) ~ Type, data = dicecoin, dist = "exponential")
summary(dc.aft)
# Call:
# survreg(formula = Surv(Time, Death) ~ Type, data = dicecoin, 
#     dist = "exponential")
#             Value Std. Error    z        p
# (Intercept) 0.672      0.102 6.62 3.56e-11
# TypeDie     1.097      0.151 7.29 3.21e-13
# 
# Scale fixed at 1 
# 
# Exponential distribution
# Loglik(model)= -386.5   Loglik(intercept only)= -412.6
# 	Chisq= 52.22 on 1 degrees of freedom, p= 5e-13 
# Number of Newton-Raphson Iterations: 4 
# n= 200 

### Again like regrssion, you see the intercept and coefficient, along with test statistics and P-values
#   Note: As there is no separate scale parameter for the exponential distribution, it is fixed at 1. 
#         With other distributions you will see an estimate of the baseline scale

### Given that we know how the parameters of the exponential distribution relate to the hazard, 
#   it is pretty straightforward to back-calculate the hazard for the two groups.
#   (It is not always quite so simple to back-calculate the hazard, although it is always possible.
#    Take a look at the hazard functions for your favorite distributions to see how.)
#   Note: You can also use the predict() function to calculate the hazard for individuals with certain covariates
#         Type in "?predict.survreg" to find help on this function.


exp(-0.672 + 1.097*0)
# [1] 0.5106862

### This hazard is for the coins (i.e., TypeDie = 0)
#   It is quite close to the 0.5 we would expect from coin flipping!

exp(-(0.672 + 1.097*1))
# [1] 0.1705034

### This is the hazard for the dice (i.e., TypeDie = 1)
#   It, too, is pretty close to the 1/6 = 0.167 we would expect from dice rolling!


